//
//  View.m
//  Map
//
//  Created by NYU User on 10/11/10.
//  Copyright 2010 __MyCompanyName__. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <MapKit/MapKit.h>
#import "View.h"
#import "Annotation.h"
#import "AnnotationView.h"


@implementation View


- (id) initWithFrame: (CGRect) frame {
	if ((self = [super initWithFrame: frame]) != nil) {
		// Initialization code
		manager = [[CLLocationManager alloc] init];
		manager.delegate = self;
		manager.desiredAccuracy = kCLLocationAccuracyBest;	//default; uses power
		[manager startUpdatingLocation];
		
		CGRect f = CGRectMake(0.0, 0.0, frame.size.width, frame.size.height);
		view = [[MKMapView alloc] initWithFrame: f];
		view.delegate = self;
		view.mapType = MKMapTypeHybrid;	//default is MKMapTypeStandard
		view.zoomEnabled = YES;		//default; double tap and pinch
		[self addSubview: view];
	}
	return self;
}

/*
// Only override drawRect: if you perform custom drawing.
// An empty implementation adversely affects performance during animation.
- (void) drawRect: (CGRect) rect {
	// Drawing code
}
*/

- (void) dealloc {
	view.delegate = nil;
	[view release];
	[manager release];
	[super dealloc];
}

#pragma mark -
#pragma mark method of protocol CLLocationManagerDelegate

- (void) locationManager: (CLLocationManager *) m
	didUpdateToLocation: (CLLocation *) newLocation
	fromLocation: (CLLocation *) oldLocation {
	
	if (m == manager) {
		horizontalAccuracy = newLocation.horizontalAccuracy;
		view.delegate = self;
		longitudinalMeters = 4 * horizontalAccuracy;
		CGSize size = self.bounds.size;
		
		view.region = MKCoordinateRegionMakeWithDistance(
			newLocation.coordinate,
			longitudinalMeters * size.height / size.width, //north-south
			longitudinalMeters		//east-west
		);
		
		[manager stopUpdatingLocation];
		Annotation *p = [[Annotation alloc]
			initWithCoordinate: newLocation.coordinate];
		
		[view addAnnotation: p];	//Tap on the red pin.
		[p release];
		
	}
}

#pragma mark -
#pragma mark methods of protocol MKMapViewDelegate

- (MKAnnotationView *) mapView: (MKMapView *) v
	viewForAnnotation: (id <MKAnnotation>) annotation {

	return [[AnnotationView alloc]
		initWithAnnotation: annotation
		reuseIdentifier: nil
		pixelsPerMeter: view.bounds.size.width / longitudinalMeters
		horizontalAccuracy: horizontalAccuracy
	];
}

@end
